/*------------------------------------   Includes   ---------------------------------------------*/
#include "gen_rnd.h"

#include <stdlib.h>
#include <time.h>
#include <assert.h>
#include <math.h>
#include <iostream>


using namespace std;

/****************************************************************************************************/
// class cRnd_Number_Generator
/****************************************************************************************************/
cRnd_Number_Generator::cRnd_Number_Generator()
{

	srand(time(NULL));
}

/*--------------------------------------------------------------------------------------------------*/
int cRnd_Number_Generator::Get_Int(int n_max)
{
	assert( n_max > 0);	
	return rand()%(n_max+1);
}

/*--------------------------------------------------------------------------------------------------*/
int cRnd_Number_Generator::Get_Int(int n_min , int n_max)
{
	assert( n_min < n_max);
	assert( n_min >= 0);
	return n_min+rand()%(n_max-n_min+1);
}

/*--------------------------------------------------------------------------------------------------*/
float cRnd_Number_Generator::Get_Float()
{	
	return ((float) rand())/((float)RAND_MAX);
}


/*--------------------------------------------------------------------------------------------------*/
static float Noise1(int x)
{
	int t;
    t = (x<<13)^x;	
	return ( 1.0 - ( (t * (t * t * 15731 + 789221) + 1376312589) & 0x7fffffff) / 1073741824.0);	
    
}

/*--------------------------------------------------------------------------------------------------*/
static float Noise2(int x,int y)
{
	int t;
	t = x+y*57;
    t = (t<<13)^t;	
	float result = ( 1.0 - ( (t * (t * t * 15731 + 789221) + 1376312589) & 0x7fffffff) / 1073741824.0);	
    return result;    	
}

/*--------------------------------------------------------------------------------------------------*/
static float SmoothedNoise_1(float x)
{
    return Noise1(x)/2.0  +  Noise1(x-1)/4.0  +  Noise1(x+1)/4.0;
}

/*--------------------------------------------------------------------------------------------------*/
static float SmoothedNoise_2(float x , float y)
{
    float corners = ( Noise2(x-1, y-1)+Noise2(x+1, y-1)+Noise2(x-1, y+1)+Noise2(x+1, y+1) ) / 16.0;
    float sides   = ( Noise2(x-1, y)  +Noise2(x+1, y)  +Noise2(x, y-1)  +Noise2(x, y+1) ) /  8.0;
    float center  =  Noise2(x, y) / 4.0;
    return corners + sides + center;
}


/*--------------------------------------------------------------------------------------------------*/
static float Cosine_Interpolate(float  a, float b, float x)
{
	float ft = x * 3.1415927;
	float f = (1 - cos(ft)) * 0.5;

	return  a*(1-f) + b*f;
}

/*--------------------------------------------------------------------------------------------------*/
static float InterpolatedNoise_1(float x)
{
      int integer_X    = (int) x;
      float fractional_X = x - integer_X;

      float v1 = SmoothedNoise_1(integer_X);
      float v2 = SmoothedNoise_1(integer_X + 1);

	  return Cosine_Interpolate(v1 , v2 , fractional_X);	  
}

/*--------------------------------------------------------------------------------------------------*/
static float InterpolatedNoise_2(float x,float y)
{
      int integer_X    = (int) x;
      float fractional_X = x - integer_X;

	  int integer_Y    = (int) y;
      float fractional_Y = y - integer_Y;
      
	  float v1 = SmoothedNoise_2(integer_X,     integer_Y);
      float v2 = SmoothedNoise_2(integer_X + 1, integer_Y);
      float v3 = SmoothedNoise_2(integer_X,     integer_Y + 1);
      float v4 = SmoothedNoise_2(integer_X + 1, integer_Y + 1);

	  float i1= Cosine_Interpolate(v1 , v2 , fractional_X);
	  float i2= Cosine_Interpolate(v3 , v4 , fractional_X);	  

	  return  Cosine_Interpolate(i1 , i2 , fractional_Y);	  
}



/*--------------------------------------------------------------------------------------------------*/
float getPerlinNoise1D(float x , int Number_Of_Octaves,float persistence)
{
      float total = 0.0;	  
      float p = persistence;
      int n = Number_Of_Octaves - 1;

	  int frequency;
	  float amplitude;
	  float to_add;

	  for(int i=0;i<=n;i++)
	  {
			frequency = pow(2,i);
			amplitude = pow(p,i);
			to_add = InterpolatedNoise_1(x * frequency) * amplitude;			
			total += to_add;
	  }    

      return total;
}
 


/*--------------------------------------------------------------------------------------------------*/
float getPerlinNoise2D(float x,float y, int Number_Of_Octaves , float persistence, int offset_octave )
{
      float total = 0.0;	  
      float p = persistence;
      int n = Number_Of_Octaves - 1;

	  int frequency;
	  float amplitude;
	  float to_add;

	  for(int i=0+offset_octave;i<=n+offset_octave;i++)
	  {
			frequency = pow(2,i);
			amplitude = pow(p,i);
			to_add = InterpolatedNoise_2(x * frequency,y * frequency) * amplitude;			
			total += to_add;
	  }    
      return total;
}