#include <assert.h>
#include "terrain.h"
#include <iostream>
#include "gen_rnd.h"
#include "mytools_spline.h"
using namespace std;
using namespace mytools_spline;


//#define BI_LINEAIRE

Terrain::Terrain( unsigned int x , unsigned int y ,  unsigned int res ,  unsigned int alti_max)
{	
	assert(x>1);
	assert(y>1);
	assert(res>0);
	
	resolution = res;	
	nx = (res*(x-1)+1);
	ny = (res*(y-1)+1);	
	altitudes = NULL;
	altitudes = new float[nx*ny];
	assert( altitudes != NULL);

	cRnd_Number_Generator Rnd;
	// les points   dterminer au hasard
	int * points = new int[x*y];
	for(unsigned int i=0; i < x ; i++)	
		for(unsigned int j=0; j < y ; j++)
		{
			//points[i+j*x] = Rnd.Get_Int(0,(int)alti_max)/2;						
			points[i+j*x] = getPerlinNoise2D(i,j,2,0.5f)*alti_max;
			cout << "rnd = " << points[i+j*x] << "\n";
		}

	for( i=0; i < x ; i++)	
		for(int j=0; j < y ; j++)
		{
			cout << "p =" << points[i+j*x] << "\n";
		}

	// on interpole les altitudes entre les points pris au hasard
	for( i=0; i < x-1 ; i++)	
		for(int j=0; j < y-1 ; j++)
		{
			FillPatch(i,j,points,x,y);
		}


	cout << "init terminated ! \n";

}


void Terrain::FillPatch(unsigned int i, unsigned int j, int * points , int tx,int ty)
{
#ifdef BI_LINEAIRE
	int a,b;
	int px,py;

	double z1,z2; // pour l'interpolation bi-linaire
	double alpha; // coeff d'interp linaire
	for(a=0;a<=resolution;a++)
		for(b=0;b<=resolution;b++)						
		{

			px = i*resolution+a;
			py = j*resolution+b;
			if( px % resolution != 0 || py % resolution != 0) // si on est pas ds un coin du patch
			{				
				alpha = (double)a/(double)resolution;
				z1 = (1.0-alpha)*((double)points[i+j*tx]) + alpha*((double)points[i+1+j*tx]);
				z2 = (1.0-alpha)*((double)points[i+(j+1)*tx]) + alpha*((double)points[i+1+(j+1)*tx]);
				alpha = (double)b/(double)resolution;
				altitudes[px+py*nx] = (1.0-alpha)*z1+alpha*z2;;

			}
			else // on garde la valeur du point fixe
			{		
				altitudes[px+py*nx] = points[i+a/resolution+(j+b/resolution)*tx];				
			}
			
		}
#else
		double patch[12*4]; // ligne par ligne
		bool x_last_exist = true;
		bool x_next_exist = true;
		bool y_last_exist = true;
		bool y_next_exist = true;
		double p[3];

		if( i == 0 )
		{
			x_last_exist = false;
			cout << "no last x \n";
		}
		if( i == tx-2 )
		{
			x_next_exist = false;
			cout << "no next x \n";

		}

		if( j == 0 )
		{
			cout << "no last y \n";
			y_last_exist = false;
		}
		if( j == ty-2 )
		{
			cout << "no next y \n";
			y_next_exist = false;				
		}

		int vrai_x,vrai_y;
		for(int yy=0;yy<4;yy++)
			for(int xx=0;xx<4;xx++)			
			{				
				
				patch[xx*3+yy*12]   = xx;
				patch[xx*3+yy*12+1] = yy;

				if( xx ==0)
				{
					if( !x_last_exist )				
						vrai_x = i+xx;				
					else
						vrai_x = i+xx-1;
				}
				else if( xx == 3)
				{
					if( !x_next_exist)
						vrai_x = i+xx-1-1;
					else
						vrai_x = i+xx-1;
				}
				else
					vrai_x = i+xx-1;

				if( yy ==0)
				{
					if( !y_last_exist )				
						vrai_y = j+yy;				
					else
						vrai_y = j+yy-1;
				}
				else if( yy == 3)
				{
					if( !y_next_exist)
						vrai_y = j+yy-1-1;
					else
						vrai_y = j+yy-1;
				}
				else
					vrai_y = j+yy-1;
				
				
				cout << "ind = " << xx*3+yy*12 << "\n";
				cout << "point copi :" << vrai_x << "," << vrai_y << "\n";
				
				patch[xx*3+yy*12+2] = points[ vrai_x + vrai_y*tx];
			}			
			

		cout << "patch to interpolate : " << "\n";
		cout << "###############################################\n";
		for( yy=0;yy<4;yy++)
		{
			for(int xx=0;xx<4;xx++)	
				cout <<  patch[xx*3+yy*12] << "," <<patch[xx*3+yy*12+1] << "   |   ";
			cout << "\n";
		}
		cout << "###############################################\n";

		int px,py;
		for(int a=0;a<=resolution;a++)
			for(int b=0;b<=resolution;b++)						
			{
				

				px = i*resolution+a;
				py = j*resolution+b;
				if( px % resolution != 0 || py % resolution != 0) // si on est pas ds un coin du patch
				{	
					/*
					Catmull_Get_Point_2D( (double)a/(double)resolution , (double)b/(double)resolution , 
									   &patch[0] ,&patch[4] , &patch[8], &patch[12], p);
					*/

					
					
					Catmull_Get_Point_2D( (double)a/(double)resolution , (double)b/(double)resolution ,
									   &patch[0] ,&patch[12] , &patch[24], &patch[36], p);
					
					
					altitudes[px+py*nx] = p[2];
				}
				else // on garde la valeur du point fixe
				{		
					altitudes[px+py*nx] = points[i+a/resolution+(j+b/resolution)*tx];				
				}
				
			}

#endif

}


float Terrain::getAlti(unsigned int x, unsigned int y)
{
	assert( x>=0);
	assert( y>=0);
	assert( x < nx);
	assert( y < ny);
	return altitudes[x+y*nx];
}

void Terrain::Print()
{
	for(unsigned int y=0; y < ny ; y++)
	{
		for(unsigned int x=0; x < nx ; x++)
			cout << getAlti(x,y) << "  ";
		cout << "\n";
	}

}