#include "perlin_texture.h"
#include "../my_common/gen_rnd.h"

#include <fstream>

using namespace std;
#include "../my_common/bmp_io.h"

#define STRETCH 20.0f

LPDIRECT3DTEXTURE9 createPerlinTexture(LPDIRECT3DDEVICE9 pDevice , int tx , int ty)
{
	
	LPDIRECT3DTEXTURE9 texture;	
    if( FAILED(  pDevice->CreateTexture( tx, ty,
                                         1, 0,
                                         D3DFMT_X8R8G8B8,
                                         D3DPOOL_MANAGED,
                                         &texture,
                                         NULL ) ))
	{
		MessageBox(NULL, "Could not create perlin texture", "Textures.exe", MB_OK);
        return NULL;        
	}

	D3DLOCKED_RECT lockedRect;
	ZeroMemory(&lockedRect, sizeof(lockedRect));
	HRESULT hr = texture->LockRect(0,&lockedRect,NULL,0);
	if( FAILED(hr))
	{
		MessageBox(NULL, "Could not lock perlin texture", "Textures.exe", MB_OK);
        return NULL;        
	}

	float perlin_noise;
	unsigned char val;
	unsigned char * pBits = (unsigned char *)lockedRect.pBits;
	for(int x=0; x <tx ; x++)
		for(int y=0; y <ty ; y++)
		{
			
			perlin_noise = getPerlinNoise2D(((float)x)/STRETCH,((float)y)/STRETCH,5,0.6f,0);
			pBits[(x+y*tx)*4] = (unsigned char)((perlin_noise/2.0+0.5)*256.0);   // red
			pBits[(x+y*tx)*4+1] = 0; // green						
			pBits[(x+y*tx)*4+2] = 0;;     // blue
			pBits[(x+y*tx)*4+3] = 0;
		}
	

	texture->UnlockRect(0);	

	return texture;

}


#include <assert.h>


bool writePerlinBitmap(char * filename, int tx , int ty)
{
	unsigned char * red=NULL;
	unsigned char * green;
	unsigned char * blue;

	red = new unsigned char[tx*ty];
	assert(red!=NULL);
	green = new unsigned char[tx*ty];
	assert(green!=NULL);
	blue = new unsigned char[tx*ty];
	assert(blue!=NULL);

	
	float perlin_noise;		
	for(int y=0; y <ty ; y++)
		for(int x=0; x <tx ; x++)
		{
			perlin_noise = getPerlinNoise2D(((float)x)/STRETCH,((float)y)/STRETCH,2,0.6f);
			blue[x+y*tx] = (perlin_noise/2.0+0.5)*256.0;
			green[x+y*tx] = 0;
			red[x+y*tx] = 0;
		}

	bool result = bmp_write(filename,tx,ty,red,green,blue);
	delete red;
	delete green;
	delete blue;

	return !result;
}